/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */

#ifndef BASE_DBM_H
#define BASE_DBM_H

#include "rdbm.const"

/* Class definition: access methods for RDBM and Tie, available in every
   instantiation.

   put = enter key/data pair (will overwrite existing entry if any)
   put_new = put, but returns an error if there's already an entry for the key
   append = put, but appends to existing entry if any
   del = delete
   get = lookup value for key
   seq = firstkey .. nextkey (call it with "" to get first()
   sync = sync cache with DBM on disc (the RDBM Client's destructor
	automatically does this if del(), put() or seq() has been called)

  All the int functions return 0 for success

*/

template <class T>
class BASE_DBM {
protected:
  virtual int file_d() const = 0 ;
  virtual void* dbm_d() const = 0 ;
  virtual bool always_locked() const = 0 ;
public:
  virtual int sync() =0 ;
  virtual int del(const char*) =0 ;
  virtual int put(const char*, const T&) =0 ;
  virtual int put_new(const char*, const T&) =0 ;
  virtual T get(const char*) =0 ;
  virtual const char* seq(const char*) =0 ;

  virtual int fetch(const char* key) { return ! get(key) ; }
  virtual int nextkey(const char* key) { return ! seq(key) ; }
  virtual int firstkey() { return nextkey("") ; }

  virtual int errnum() const = 0 ;
  virtual const char* errmsg() const = 0 ;
  virtual const char* errmsg1(const int) const ;

/* clear_err() is dbm-dependent: use with caution if you're
   writing portable code
*/
  virtual void clear_err() {}

  virtual ~BASE_DBM() {}

} ;

/* Put htis here, 'cos the implementations share these.
*/

template <class T>
inline const char* BASE_DBM<T>::errmsg1(const int e) const {
  switch (e) {
  case 0:
    return "" ;
  case RDBM_NO_KEY:
    return "No key" ;
  case RDBM_KEY_TOO_LONG:
    return "Key too long (change this in rdbm.const)" ;
  case RDBM_VAL_TOO_LONG:
    return "Data too big (change this in rdbm.const)" ;
  case RDBM_DBM_FAILED:
    return "DBM operation failed" ;
  case RDBM_DBM_LOCKED:
    return "DBM locked by another process (please try again)" ;
  case RDBM_NO_DATA:
    return "No data found at this point in DBM" ;
  case RDBM_OPEN_FAILED:
    return "Can't open DBM." ;
  default:
    return errmsg() ;	// don't let errmsg() call errmsg1
  }
}

#endif
