/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */

#include "rdbm.const"

#ifndef DATUM_H
#define DATUM_H

/* MUST follow after something which defines:
	base_datum
	DB_USES_MALLOC
	dptr
	dsize

	The fields used here are right for NDBM and GDBM.
	We use the preprocessor to work with Berkeley DB 1.85.

	DB 2.0 has a bigger datum that needs its own definition,
	so we use virtual methods where appropriate (we hack that
	by #defining DATUM before including this file)

	GDBM (and NDBM if it's GDBM in disguise) mallocs memory which
	we have to free.   #defining DB_USES_MALLOC tells the DATUM
	class whether to do this.   Berkeley DB makes this optional;
	we use the (presumably more efficient) USERMEM option.
*/

typedef enum { DATUM_NULL, DATUM_WANT_FREE, DATUM_READONLY } datum_status ;

class DATUM : public base_datum {

#ifdef DB_USES_MALLOC
#define usermem 0
#else
#define DB_USES_MALLOC 0
  char usermem[rdbm_max_data] ;
#endif

private:
  const datum_status status ;
 
public:
  DATUM(const char* data_in) : status(DATUM_READONLY) {
	dptr = (char*) data_in ;
	dsize = strlen(data_in) ;
  }
  DATUM() : status(DB_USES_MALLOC?DATUM_WANT_FREE:DATUM_NULL) {
	dptr = usermem ;
	dsize = sizeof(usermem) ;
  }
#if DB_USES_MALLOC
  DATUM(const base_datum& d) : status(DATUM_WANT_FREE) {
	dptr = d.dptr ;
	dsize = d.dsize ;
  }
    
  virtual DATUM& operator=(const base_datum& d) {
    if ( status != DATUM_READONLY ) {
	dptr = d.dptr ;
	dsize = d.dsize ;
    }
    return *this ;
  }
#else
  DATUM(const base_datum& d) : status(DATUM_NULL) {
    dptr = d.dptr ;
    dsize = d.dsize ;
  }
  virtual DATUM& operator=(const base_datum& d) {
    if ( status != DATUM_READONLY ) {
	dptr = d.dptr ;
	dsize = d.dsize ;
    }
    return *this ;
  }
#endif
  virtual ~DATUM() {
	if ( dptr && (status == DATUM_WANT_FREE) ) free(dptr) ;
  }
} ;
  
#endif
