
/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
#ifndef DBSERVER_H
#define DBSERVER_H

#include <iostream.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

template <class DBM>
class DBUser : public DBM {
public:
  DBUser() {
    if ( ! dbm_d() ) {
	cerr << "AnyDBM error opening DBM: " << errmsg() << endl ;
	exit(-1) ;
    }
/* Workaround for DB 1.85 problems - the same fix as GDBM uses!

an optimal regime would apply finer locking:
   NDBM - could do better
   GDBM - absolutely can't do better (always_locked)
   DB 1.85 - ought to be able to improve, but it has problems with sync()
	     so it breaks in practice
   DB 2.0 - could do better
   RDBM - this is optimal: the server has a lock, but unlimited clients
	  can use it without restrictions
*/
    if ( ! DBM::always_locked() ) {
	struct flock lock ;
	lock.l_type = F_WRLCK ;
	lock.l_start = 0 ;
	lock.l_whence = SEEK_SET ;
	lock.l_len = 0 ;
	lock.l_pid = getpid() ;
	if (fcntl (file_d(), F_SETLK, &lock) == -1) {
	  cerr << "AnyDBM error locking DBM: "
		<< strerror(errno) << endl ;
	  exit(-1) ;
	}
    }
  }
} ;
  // the destructors for the DBMs do the tidying up.

template <class DBM, class LOCKMGR>
class DBServer : public DBUser<DBM>, public LOCKMGR {
protected:
  virtual bool always_locked() const { return true ; }
} ;
#endif
