
/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
#ifndef DB_1_85_H
#define DB_1_85_H

#include "BASE_DBM.h"

#include <db.h>
#include <stdlib.h>
#include <fcntl.h>
#include <iostream.h>
#include <sys/file.h>

/* setup one of the modes.   Default to Hash.   Don't see why anyone
   would want to use RECNO, but include it for completeness since
   it's no extra work.

   This gives compile-time defaults, which can be overridden at
   runtime by environment variables
*/

/* Make this pure virtual; inherit HASH, BTREE, RECNO classes */
template <class T>
class DB_1_85 : public BASE_DBM<T> {

public:
  int sync() ;
  int del(const char*) ;
  int put(const char*, const T&) ;
  int put_new(const char*, const T&) ;
  T get(const char*) ;
  const char* seq(const char*) ;
  int errnum() const { return dbstatus ; }
  const char* errmsg() const ;
  void clear_err() { dbstatus = 0 ; }

protected:
  struct DB* _fd ;
  DB_1_85() { dbstatus = 0 ; }
  virtual ~DB_1_85() ;
  void* dbm_d() const { return _fd ; }
  int file_d() const { return _fd->fd(_fd) ; }
  virtual bool always_locked() const { return false ; }
  int dbstatus ;
} ;

#define base_datum DBT
#define dptr data
#define dsize size
#include "DATUM.h"
#undef dsize
#undef dptr

template <class T>
inline DB_1_85<T>::~DB_1_85() {
  if ( _fd ) {
    if ( _fd->close(_fd) == -1 )
       cerr << "DB_1_85: error closing DB" << endl ;
    _fd = 0 ;
  }
}
template <class T>
inline int DB_1_85<T>::put(const char* Key, const T& data) {
  DATUM key(Key) ;
  DATUM content(data) ;

  if (content.size >= rdbm_max_data)
    return RDBM_VAL_TOO_LONG ;
  if (key.size >= rdbm_max_key)
    return RDBM_KEY_TOO_LONG ;

  return dbstatus = _fd->put (_fd, &key, &content, 0), dbstatus ;
}
template <class T>
inline int DB_1_85<T>::put_new(const char* Key, const T& data) {
  DATUM key(Key) ;
  DATUM content(data) ;

  if (content.size >= rdbm_max_data)
    return RDBM_VAL_TOO_LONG ;
  if (key.size >= rdbm_max_key)
    return RDBM_KEY_TOO_LONG ;

  return dbstatus = _fd->put (_fd, &key, &content, R_NOOVERWRITE), dbstatus ;
}
template <class T>
inline T DB_1_85<T>::get(const char* key) {
 DATUM id(key) ;
 static char ret[rdbm_max_data] ;
 DATUM d ;
 dbstatus = _fd->get(_fd, &id, &d, 0) ;
 switch ( dbstatus ) {
  case 0:
   if ( d.size >= rdbm_max_data )
     d.size = rdbm_max_data - 1 ;
   strncpy(ret, (char*)d.data, d.size) ;
   ret[d.size] = 0 ;
   break ;
  case 1:
   ret[0] = 0 ;
   break ;
  case -1:
   return (const char*) 0 ;
 }
 return ret ;
}
template <class T>
inline int DB_1_85<T>::del(const char* key) {
 DATUM d(key) ;
  if (d.size >= rdbm_max_key)
    return RDBM_KEY_TOO_LONG ;
 return dbstatus = _fd->del(_fd, &d, 0) , dbstatus ;
}
template <class T>
inline int DB_1_85<T>::sync() {
  return dbstatus = _fd->sync(_fd, 0), dbstatus ;
}
template <class T>
inline const char* DB_1_85<T>::seq(const char* x) {
  static char ret[rdbm_max_key] ;
  DATUM id ;
  DATUM nextkey ;

  dbstatus = strlen(x)
	? _fd->seq(_fd, &id, &nextkey, R_NEXT)
	: _fd->seq(_fd, &id, &nextkey, R_FIRST) ;

  switch (dbstatus) {
  case -1:
    return 0 ;
  case 0:
    if ( id.size >= rdbm_max_key )
      id.size = rdbm_max_key - 1 ;
    strncpy(ret, (char*)id.data, id.size) ;
    ret[id.size] = 0 ;
    break ;
  default:
    ret[0] = 0 ;
  }

  return ret ;
}
template <class T>
inline const char* DB_1_85<T>::errmsg() const {
  switch (dbstatus) {
    case RET_ERROR: return "Error in DB operation." ;
    case RET_SUCCESS: return "" ;
    case RET_SPECIAL: return "No such entry" ;
  }
  return "Internal Error - impossible DB error code." ;
}

#endif
