/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */

#ifndef GDBM_H
#define GDBM_H

#ifndef RDBM_GDBM_FILE
#define RDBM_GDBM_FILE "/var/AnyDBM/rdbm.gdbm"
#endif

#include "BASE_DBM.h"

#include <gdbm.h>

template <class T>
class GDBM : public BASE_DBM<T> {
private:
  GDBM_FILE gdbm_fd ;
protected:
  void* dbm_d() const { return gdbm_fd ; }
  int file_d() const { return -1 ; }		// dummy
  bool always_locked() const { return true ; }
public:
  int sync() ;
  int del(const char*) ;
  int put(const char*, const T&) ;
  int put_new(const char*, const T&) ;
  T get(const char*) ;
  const char* seq(const char*) ;
  GDBM() ;
  virtual ~GDBM() ;
  int errnum() const { return gdbm_errno ; }
  virtual const char* errmsg() const { return gdbm_strerror(gdbm_errno) ; }

} ;

#define base_datum datum
#define DB_USES_MALLOC 1
#include "DATUM.h"

#include "rdbm.const"
#include <iostream.h>

template <class T>
inline GDBM<T>::GDBM() {
  char* name = getenv("ANYDBM_DB") ;
  if ( !name ) name = RDBM_GDBM_FILE ;
  gdbm_fd = gdbm_open (name, 0, GDBM_WRCREAT|GDBM_FAST, 0600, 0) ;
}
template <class T>
inline GDBM<T>::~GDBM() {
  if (gdbm_fd) gdbm_close(gdbm_fd) ;
}
template <class T>
inline int GDBM<T>::put(const char* Key, const T& data) {
  DATUM key(Key), content(data) ;

  if (content.dsize >= rdbm_max_data)
    return RDBM_VAL_TOO_LONG ;
  if (key.dsize >= rdbm_max_key)
    return RDBM_KEY_TOO_LONG ;

  return gdbm_store (gdbm_fd, key, content, GDBM_REPLACE) ;
}
template <class T>
inline int GDBM<T>::put_new(const char* Key, const T& data) {
  DATUM key(Key), content(data) ;

  if (content.dsize >= rdbm_max_data)
    return RDBM_VAL_TOO_LONG ;
  if (key.dsize >= rdbm_max_key)
    return RDBM_KEY_TOO_LONG ;

  return gdbm_store (gdbm_fd, key, content, GDBM_INSERT) ;
}
template <class T>
inline T GDBM<T>::get(const char* key) {
  DATUM id(key) ;
  static char ret[rdbm_max_data] ;
  DATUM d = gdbm_fetch(gdbm_fd, id) ;
  if ( d.dsize >= rdbm_max_data )
    d.dsize = rdbm_max_data - 1  ;
  strncpy(ret, d.dptr, d.dsize) ;
  ret[d.dsize] = 0 ;
  return ret ;
}
template <class T>
inline int GDBM<T>::del(const char* key) {
  DATUM d(key) ;
  if (d.dsize >= rdbm_max_key)
    return RDBM_KEY_TOO_LONG ;
  return gdbm_delete(gdbm_fd, d) ;
}
template <class T>
inline int GDBM<T>::sync() {
  gdbm_sync(gdbm_fd) ;
  return 0 ;
}
template <class T>
inline const char* GDBM<T>::seq(const char* x) {
  static char ret[rdbm_max_key] ;
  DATUM id(x) ;
  DATUM nextkey ;
  if ( !id.dsize )
    nextkey = gdbm_firstkey(gdbm_fd) ;
  else
    nextkey = gdbm_nextkey(gdbm_fd, id) ;

  if ( !nextkey.dptr )
    ret[0] = 0 ;
  else {
    if ( nextkey.dsize >= rdbm_max_key )
      nextkey.dsize = rdbm_max_key - 1 ;
    strncpy(ret, nextkey.dptr, nextkey.dsize) ;
    ret[nextkey.dsize] = 0 ;
  }

  return ret ;
}

#endif
