
/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
#include "LOCKMGR.h"

/*********************************************************
Locking functions: Lock, Unlock, Check

These are required for listing a dbm (seq()):

- with NDBM, they are absolutely necessary if you ever want to list the DBM
- with GDBM you can reasonably dispense with them at the cost of possible
  minor glitches in listings.
- With Berkeley DB, the cursor permits get/put without locking, but
  two processes using seq() would interfere (as with NDBM).   Use
  CURSOR locking if you want listing.
- Berkeley DB 2.0's multiple cursors are not (yet) supported

LOCKMGR uses env variable RDBM_LOCK to determine how to do this.
"FULL" locking always permits safe listing.   Other values are "NONE" (if you
never want to list the DBM, or with GDBM), and "CURSOR" (for Berkeley DB).

*********************************************************/

DBM_TIMER_LOCK::DBM_TIMER_LOCK( const int t) : _timeout(t) {
  _lock.pid = _lock.host = 0 ;
}
const DBM_LOCK DBM_TIMER_LOCK::has_lock(const rdbm_ident_t& l) const {
 return
    (_lock.pid==0) && (_lock.host==0)
	? NO_LOCK
	:    (_lock.pid==l.pid) && (_lock.host==l.host)
		? OUR_LOCK
		: OTHER_LOCK ;
}
const int DBM_TIMER_LOCK::lock_timeout() const {
 int result = (time(0) - locktime) > _timeout ;
 return result ;
}
DBM_LOCK DBM_TIMER_LOCK::locked(const rdbm_ident_t& l) {

 DBM_LOCK ret = has_lock(l) ;
 if (ret==OTHER_LOCK)
   if ( lock_timeout()) {
     _lock.pid = _lock.host = 0 ;
     ret = NO_LOCK ;
   }
  return ret ;
}
DBM_LOCK DBM_TIMER_LOCK::lock(const rdbm_ident_t& l) {

 DBM_LOCK ret = locked(l) ;
 switch (ret) {
  case NO_LOCK: 	// not locked or stale lock
   _lock = l ;		// claim it ourselves
   ret = OUR_LOCK ;

  case OUR_LOCK:
   locktime = time(0) ;	// reset timeout - we're (still) using it.

  case OTHER_LOCK:	// can't do anything
	;
 }
 return ret ;
}
DBM_LOCK DBM_TIMER_LOCK::unlock(const rdbm_ident_t& l) {

 DBM_LOCK ret = locked(l) ;

 if (ret == OUR_LOCK) {		// disclaim lock if ours
  _lock.pid = _lock.host = 0 ;
  ret = NO_LOCK ;
 }

 return ret ;
}
