
/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
#ifndef LOCKMGR_H
#define LOCKMGR_H

#include "rdbm.h"

/* These locking functions are required for listing a dbm (seq()).
   They have nothing to do with database integrity, which we get
   for free by making this an RPC server.

   See comments in LOCKMGR.cc for use of the RDBM_LOCK environment var.
*/

typedef enum { NO_LOCK, OUR_LOCK, OTHER_LOCK } DBM_LOCK ;
typedef enum { RDBM_NO_LOCK, RDBM_CURSOR_LOCK, RDBM_FULL_LOCK } DBM_LOCK_TYPE ;

class DBM_LOCK_BASE {	// Null locking regime
public:
  virtual DBM_LOCK lock(const rdbm_ident_t&) { return OUR_LOCK ; }
  virtual DBM_LOCK unlock(const rdbm_ident_t&) { return NO_LOCK ; }
  virtual DBM_LOCK locked(const rdbm_ident_t&) { return OUR_LOCK ; }
} ;


class DBM_TIMER_LOCK : public DBM_LOCK_BASE {	// timed lock for listing

private:
  const int _timeout ;
  rdbm_ident_t _lock ;
  time_t locktime ;

  virtual const DBM_LOCK has_lock(const rdbm_ident_t&) const ;
  virtual const int lock_timeout() const ;
  
public:
  virtual DBM_LOCK lock(const rdbm_ident_t&) ;
  virtual DBM_LOCK unlock(const rdbm_ident_t&) ;
  virtual DBM_LOCK locked(const rdbm_ident_t&) ;
  DBM_TIMER_LOCK(const int t =2) ;
} ;

#endif
