
/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
#ifndef NDBM_H
#define NDBM_H

#ifndef RDBM_NDBM_FILE
#define RDBM_NDBM_FILE "/var/AnyDBM/rdbm.ndbm"
#endif

#include "BASE_DBM.h"

#include <ndbm.h>

template <class T>
class NDBM : public BASE_DBM<T> {
private:
  DBM* _dbm ;
  int dbstatus ;
protected:
  void* dbm_d() const { return _dbm ; }
  int file_d() const { return dbm_pagfno(_dbm) ; }
  virtual bool always_locked() const { return false ; }
public:
  int sync() ;
  int del(const char*) ;
  int put(const char*, const T&) ;
  int put_new(const char*, const T&) ;
  T get(const char*) ;
  const char* seq(const char*) ;
  NDBM() ;
  virtual ~NDBM() { if (_dbm) dbm_close(_dbm) ; }
  int errnum() const { return dbstatus ; }
  const char* errmsg() const
	{ return errnum() ? "Error in DBM operation" : "" ; }
  void clear_err() { dbstatus = 0 ; }
} ;

#define base_datum datum

// use 1 if it's GDBM pretending to be NDBM
#define DB_USES_MALLOC 0
#define base_datum datum
#include "DATUM.h"

#include "rdbm.const"
#include <iostream.h>
#include <sys/file.h>
#include <fcntl.h>

template <class T>
inline NDBM<T>::NDBM() {
  char* name = getenv("ANYDBM_DB") ;
  if ( !name ) name = RDBM_NDBM_FILE ;
  if (_dbm = dbm_open (name , O_CREAT|O_RDWR, 0600), _dbm != 0)
    dbstatus = 0 ;
  else
    dbstatus = RDBM_OPEN_FAILED ;
}
template <class T>
inline int NDBM<T>::put(const char* Key, const T& data) {
  DATUM key(Key), content(data) ;
  if (content.dsize >= rdbm_max_data)
    return dbstatus = RDBM_VAL_TOO_LONG, dbstatus ;
  if (key.dsize >= rdbm_max_key)
    return dbstatus = RDBM_KEY_TOO_LONG, dbstatus ;
  dbstatus = dbm_store (_dbm, key, content, DBM_REPLACE) ;
  return dbstatus ;
}
template <class T>
inline int NDBM<T>::put_new(const char* Key, const T& data) {
  DATUM key(Key), content(data) ;
  if (content.dsize >= rdbm_max_data)
    return dbstatus = RDBM_VAL_TOO_LONG, dbstatus ;
  if (key.dsize >= rdbm_max_key)
    return dbstatus = RDBM_KEY_TOO_LONG, dbstatus ;
  return dbstatus = dbm_store (_dbm, key, content, DBM_INSERT), dbstatus ;
}
template <class T>
inline T NDBM<T>::get(const char* key) {
  DATUM id(key) ;
  static char ret[rdbm_max_data] ;
  DATUM d  = dbm_fetch(_dbm, id) ;
  if ( d.dsize >= rdbm_max_data )
    d.dsize = rdbm_max_data - 1 ;
  strncpy(ret, d.dptr, d.dsize) ;
  ret[d.dsize] = 0 ;
  return ret ;
}
template <class T>
inline int NDBM<T>::del(const char* key) {
  DATUM d(key) ;
  return dbstatus = dbm_delete(_dbm, d), dbstatus ;
}
template <class T>
inline int NDBM<T>::sync() {
  return 0 ;
}
template <class T>
inline const char* NDBM<T>::seq(const char* x) {
  static char ret[rdbm_max_key] ;
  DATUM id(x) ;
  DATUM nextkey ;
  if ( !id.dsize )
    nextkey = dbm_firstkey(_dbm) ;
  else
    nextkey = dbm_nextkey(_dbm) ;

  if ( !nextkey.dptr ) {
    dbstatus = RDBM_NO_DATA ;
    ret[0] = 0 ;
  } else {
    if ( nextkey.dsize >= rdbm_max_key )
      nextkey.dsize = rdbm_max_key - 1 ;
    strncpy(ret, nextkey.dptr, nextkey.dsize) ;
    ret[nextkey.dsize] = 0 ;
  }
 
  return ret ;
}


#endif
