
/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
#ifndef RDBM_C_H
#define RDBM_C_H

/* This is for C and is untested (but trivial).   The other file RDBM.h
   is for C++ and is tested.

   Include one, NOT both of them, in programs using RDBM!
*/

#include "rdbm.const"

#ifdef RDBM_CC_H
typedef RDBM<const char*>* rdbm_fd ;
extern "C" {
#else
typedef void* rdbm_fd ;
#endif

/* rdbm_open() is a constructor.  Always check errnum(), and rdbm_close()
   it when done even if the rdbm_open() failed
*/
rdbm_fd rdbm_open(const char* host, const int prog, const int vers) ;
void rdbm_close(rdbm_fd fd) ;

/* zero for success - else errnum() */
int rdbm_reconnect(rdbm_fd fd, const char* host, const int prog, const int vers);
int rdbm_put_new(rdbm_fd fd, const char* key, const char* val) ;
int rdbm_put(rdbm_fd fd, const char* key, const char* val) ;
int rdbm_del(rdbm_fd fd, const char* key) ;
int rdbm_fetch(rdbm_fd fd, const char* key) ;
int rdbm_nextkey(rdbm_fd fd, const char* key) ;
int rdbm_firstkey(rdbm_fd fd) ;

/* zero for failure - check errnum() */
const char* rdbm_get(rdbm_fd fd, const char* key) ;
const char* rdbm_seq(rdbm_fd fd, const char* key) ;

int rdbm_errnum(rdbm_fd fd) ;
void rdbm_clearerr(rdbm_fd fd) ;
const char* rdbm_errmsg(rdbm_fd fd) ;
const char* rdbm_lastval(rdbm_fd fd) ;

#ifdef RDBM_CC_H
}
#endif

#endif
