
/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
// For #inclusion from Tie.h: don't use directly

template <class DBM, class T>
class TiedArray {
  friend class TiedObject<DBM, T> ;
private:
  DBM& handle ;
  bool _eof ;
public:
  TiedArray(DBM& dbm) throw (dbmexception) : handle(dbm), _eof(false)
	{ if (dbm.errnum()) throw(dbmexception(dbm.errnum())) ; }
  ~TiedArray() {}

// we want some nice convenient associative array and output-iterator notation
  TiedObject<DBM,T> operator[](const char*) throw (dbmexception) ;
  TiedObject<DBM,T> operator*() throw(dbmexception) ;


/* Add the BASE_DBM functions.  Might be better to inherit, but then we
   get double-destructor of the DBM, which means either making things
   static (bad) or some painful messing about.  Maybe in a future release ..
*/

  int sync() { return handle.sync() ; }
  int del(const char* x) { return handle.del(x) ; }
  int put(const char* x, const T& v) { return handle.put(x,v) ; }
  int put_new(const char* x, const T& v) { return handle.put_new(x,v) ; }
  T get(const char* x) { return handle.get(x) ; }
  const char* seq(const char* x) { return handle.seq(x) ; }
  int fetch(const char* x) { return handle.fetch(x) ; }
  int nextkey(const char* x) { return handle.nextkey(x) ; }
  int firstkey() { return handle.firstkey() ; }
  int errnum() { return handle.errnum() ; }
  const char* errmsg() { return handle.errmsg() ; }

  bool eof() const { return _eof ; }
} ;
template <class DBM, class T>
inline TiedObject<DBM,T> TiedArray<DBM,T>::operator[](const char* x) {
  if (!x)
    throw dbmexception(RDBM_NO_KEY) ;
  if (!x[0])
    throw dbmexception(RDBM_NO_KEY) ;
  if (strlen(x) > rdbm_max_key)
    throw dbmexception(RDBM_KEY_TOO_LONG) ;
  return TiedObject<DBM,T>(this,x) ;
}
template <class DBM, class T>
inline TiedObject<DBM,T> TiedArray<DBM,T>::operator*() {
  const char* key = handle.seq("") ;
  if (key)
    if (*key)
      return _eof=false, operator[](key) ;
/* DBM is empty ... still have to return something */
  return _eof=true, TiedObject<DBM, T>(this, "") ;
}
