/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
// For #inclusion from Tie.h: don't use directly

template <class DBM, class T>
class TiedObject {
  friend class TiedArray<DBM, T> ;
private:
  class TiedArray<DBM, T>* _dbm ;
//  const char* _key ;
  char _key[rdbm_max_key] ;
  T value ;
  TiedObject() ;
protected:
  TiedObject(class TiedArray<DBM,T>* dbm, const char* key)
	: _dbm(dbm) , value(0) { strcpy(_key, key?key:"") ; }
public:
  ~TiedObject() { }
  operator const T&() throw (dbmexception) ;			// lookup
  const T& operator=(const T&) throw (dbmexception) ;		// set
  T& operator=(T&) throw (dbmexception) ;			// set
  int operator=(const NullTiedObject) throw (dbmexception) ;	// delete

  T& operator*() throw (dbmexception) ;				// lookup
  TiedObject& operator++() ;
  TiedObject& operator++(int) ;

  const char* key() const { return _key ; }
  T& data() throw (dbmexception) ;				// lookup
} ;
template <class DBM, class T>
T& TiedObject<DBM, T>::operator*() { return data() ; }
  
template <class DBM, class T>
inline TiedObject<DBM,T>& TiedObject<DBM,T>::operator++() {
  const char* nextkey = _dbm->handle.seq(_key) ;
  if (nextkey)
    if (*nextkey)
      *this = (*_dbm)[nextkey] ;
    else
      _dbm->_eof = true ;
  else
    _dbm->_eof = true ;
  return *this ;
}
template <class DBM, class T>
inline TiedObject<DBM,T>& TiedObject<DBM,T>::operator++(int) {
  static TiedObject<DBM,T> tmp ;		// this 
  tmp._dbm = _dbm ;
  strcpy(tmp._key,_key) ;
  ++*this ;
  return tmp ;
}

template <class DBM, class T>
inline int TiedObject<DBM,T>::operator=(const NullTiedObject) {
  int status = _dbm->handle.del(_key) ;
  if (status)
    throw dbmexception(status) ;
  return status ;
}
template <class DBM, class T>
inline T& TiedObject<DBM,T>::operator=(T& val) {
  int status = _dbm->handle.put(_key, val) ;
  if (status)
    throw dbmexception(status) ;
  return val ;
}
template <class DBM, class T>
inline const T& TiedObject<DBM,T>::operator=(const T& val) {
  int status = _dbm->handle.put(_key, val) ;
  if (status)
    throw dbmexception(status) ;
  return val ;
}
template <class DBM, class T>
inline T& TiedObject<DBM, T>::data() {
  if (!value)
    value = _dbm->handle.get(_key) ;
  if ( !value ) {
    int err = _dbm->handle.errnum() ;
    if (err)
      throw dbmexception(err) ;
  }
  return value ;
}
template <class DBM, class T>
inline TiedObject<DBM, T>::operator const T& () {
  return data() ;
}
