/*
	To build as old-fashioned CGI:
	c++ -Wall -O2 rdbmmanage.cc -o rdbmmanage.cgi -lrdbm -lcgi++ -lg++

	To build as embedded CGI:
	c++ -Wall -O2 -DECGI -shared rdbmmanage.cc \
		-o rdbmmanage.so -lrdbm -lcgi++ -lg++
	(note this works with GNU ld; on non-GNU YMMV)

	(you might also need -lnsl in either case)

	The rdbm and cgi++ libraries are from the AnyDBM and CGI++
	packages, available free from <URL:http://www.webthing.com/>
	under the GNU LGPL.
*/

#define RDBMMANAGE_PAGE "rdbmmanage.shtml"

#include <stdlib.h>
#include <unistd.h>
//#include <crypt.h>
#include <CGI++/CGI.h>
#include <time.h>
#include <AnyDBM/RDBM.h>

CGI cgi ;

unsigned char randchar() {
  unsigned char c = rand() % 62 ;
  if ( c < 10 )
    return c + '0' ;
  if ( c < 36)
    return c + 'A' - 10 ;
  return c + 'a' - 36 ;
}

bool pw_report (const char* user, const String& pw,
		const String& hash, const String& gp) {
  return cout <<
	"<h2>Updated User Database OK</h2>"
	"<p>Your record is now:<table>"
	"<tr><th>Username</th><td>" << user << "</td></tr>"
	"<tr><th>Group</th><td>" << gp << "</td></tr>"
	"<tr><th>Password</th><td>" << pw << " (encrypted to "
		<< hash << ")</td></tr>"
	"</table>"
	"<p>We <strong>strongly recommend</strong> you save this page "
	"(or mail it to yourself) to remind you of your details.</p>"
  , true ;
}
bool pw_delete (const char* user) {
  return cout <<
	"<h2>Deleted " << user << " OK</h2><p>Goodbye!</p>"
  , true ;
}
bool pw_error (const char* msg, const char* msg2 ="") {
  return cout <<
	"<h2>Error</h2><p>Operation failed - error message was <tt>"
	<< msg << msg2 << "</tt></p>"
	"<p>This could mean:<ul>"
	"<li>The RDBM server is not running, or is unreachable by the webserver."
	"<li>The data you entered is not acceptable (e.g. no username)."
	"<li>The server is misconfigured.  I hope that's because you're "
	"busy setting it up right now!"
	"</ul><p>Please try again, and "
	"<a href=\"mailto:" << cgi.server_admin() << "\">contact the "
	"server administrator</a> if the problem persists.</p>"
  , false ;
}
bool pw_update() {
  String& name = cgi["name"].as_text() ;
  unsigned int trunc = cgi["trunc"].as_int() ;
  unsigned int digits = cgi["digits"].as_int() ;
  String& password = cgi["password"].as_text() ;
  String& action = cgi["action"].as_text() ;
  String& group = cgi["group"].as_text() ;

  const char* username = 0 ;

  Regex strip("[^A-Za-z0-9_]") ;
  password.gsub(strip,"") ;
  group.gsub(strip,"") ;

  RDBM<const char*> db ;
  if (db.errnum())
    return pw_error(db.errmsg()) ;
  

/* Update and Delete require user to be logged in as themself */
  if ( ( action == "update" ) || ( action == "delete" ) ) {
    username = cgi.remote_user() ;
    if ( !strlen(username) )
	return pw_error("You must be logged in as the user whose record you wish to modify or delete.") ;
    if ( action == "update" && !password.length() )
	return pw_error("You must specify password when updating a record.") ;

/* adding a new user - process name and password as necessary */
  } else {					// new user
    name.gsub(strip,"") ;
    name = downcase(name) ;
    if (trunc && (name.length() > trunc) )
      name = name.at(0,trunc) ;

    if ( !name.length() )
      return pw_error ("Invalid Username") ;

    if ( digits || !password.length() )		// anything need rand ?
      srand(time(0)) ;

    if ( !password.length() )			// generate random 5-digit
      do {
	ostrstream o ;
	o << (rand()%100000) << ends ;
	password = o.str() ;
      } while (password.length() < 5) ;

    if ( digits ) {
      int base = 10 ;
      while (--digits)
	base *= 10 ;
      for (int tries = 0; tries < base/5; ++tries) {
				// ensure unique name
				// limit #tries to kill infinite loop
				// if digits inadequate
	ostrstream o ;
	o << name << (rand() % base)  << ends ;
	username = o.str() ;
	db.clear_err() ;
	if ( ! db.put_new(username, "dummy") )	// atomic test of whether
	  break ;				// username is available
      }
    } else {
      username = name ;
      db.put_new(username,"dummy") ;
    }
    if (db.errnum())
      return pw_error("Duplicates an existing entry in password database.") ;
  }

/* now we have username, password and group ... */

  if ( action == "delete" ) {
    if (db.del(username))
      return pw_error("RDBM Error - ", db.errmsg()) ;
    else
      return pw_delete(username) ;
  } else {
    char salt[3] ;
    salt[0] = randchar() ;
    salt[1] = randchar() ;
    salt[2] = 0 ;
    String hash(crypt(password, salt)) ;
    if (db.put(username, hash + ":" + group) )
      return pw_error("RDBM Error - ", db.errmsg()) ;
    else
      return pw_report(username, password, hash, group) ;
  }
  return pw_error("Unexpected execution path - bug in Password Manager Program!") ;
}

/* the following are for use in an SSI; to use as standalone will
   require a bit more HTML output :-)
*/
void head() {
  cout <<
	"Cache-Control: private\n"
	"Content-Type: text/html\n\n"
	"<hr><h1>RDBM Password Manager for Apache</h1>"
  ;
}
void foot() {
  cout <<
	"<p><a href=\"http://www.webthing.com/software/anydbm/apache.html#cgi\">More information.</a></p><hr>"
  ;
}
void pw_showform() {
  cgi.define(5
	, "name:input::8"
	, "action:select:new"
	, "group:input:users:8"
	, "trunc:hidden:6"
	, "digits:hidden:3"
  ) ;

  cgi["action"]["new"] = "Add user" ;

  if ( cgi.remote_user().length() ) {
    cgi["action"]["delete"] = "Delete user" ;
    //cgi["action"]["update"] = "Update user record" ;
    cgi["action"].set("delete") ;
    cgi["name"].set(cgi.remote_user()) ;
  }
/* Using GET here is bad practice, but POSTing to a .shtml chokes Apache */
  cout << cgi.start(RDBMMANAGE_PAGE, "GET") << cgi.render("action")
	<< cgi.render("trunc") << " Username: " << cgi.render("name") ;
  cout << " Group: " << cgi.render("group") << cgi.render("digits")
	<< cgi.submit("OK") << cgi.end() ;
}

/* allow use as old-fashioned CGI, or as embedded CGI.  Not tested as
   ECGI with CGI++ enhancements.
*/

#ifdef ECGI
int rdbmmanage() {
#else
int main() { 
#endif

  head() ;
  if ( cgi.has_form_data() ) {
    cgi.form() ;
    pw_update() ;
  } else
    pw_showform() ;
  foot() ;
  return 0 ;
}
