
/* ===================  AnyDBM  ===============================
Copyright (C) 1998, WebThing Ltd
Author: Nick Kew

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

-------------
Author/Copyright Holder Information: please see the README file and
the AnyDBM webpages at <URL:http://www.webthing.com/software/AnyDBM/>

============================================================ */
#include "LOCKMGR.h"
#include "BASE_DBM.h"
#include "rdbm.const"
#include "rdbm.h"
#include "DBServer.h"

#if defined IMPL && defined IMPL_LOCK && defined IMPL_H
#include IMPL_H

/* Declare this top-level static to open the DBM before the rpcgen-created
   main().  This avoids the case where it registers with portmap only to
   find itself locked out of the DBM.
*/
static DBServer<IMPL<const char*>, IMPL_LOCK> _dbmgr ;
DBServer<IMPL<const char*>, IMPL_LOCK>* dbmgr() {
  return &_dbmgr ;
}
#endif

const int LOCKED = -1 ;
extern "C" {

int* rdbm_put_1_svc(rdbm_key_val_t* p, svc_req*) {
  static int ret ;
  if ( dbmgr()->locked(p->id) == OTHER_LOCK )
    ret = LOCKED ;
  else
    ret = dbmgr()->put(p->key, p->val) ? 0 : 1 ;
  return &ret ;
}
int* rdbm_put_new_1_svc(rdbm_key_val_t* p, svc_req*) {
  static int ret ;
  if ( dbmgr()->locked(p->id) == OTHER_LOCK )
    ret = LOCKED ;
  else
    ret = dbmgr()->put_new(p->key, p->val) ? 0 : 1 ;
  return &ret ;
}
char** rdbm_seq_1_svc(rdbm_key_t* p, svc_req*) {
  static char* ret ;
  if ( dbmgr()->lock(p->id) == OUR_LOCK )
    ret = (char*) dbmgr()->seq(p->key) ;
  else
    ret[0] = LOCKED ;
  return &ret ;
}
char** rdbm_get_1_svc(rdbm_key_t* p, svc_req*) {
  static char* ret ;
  if ( dbmgr()->locked(p->id) == OTHER_LOCK )
    ret[0] = LOCKED ;
  else
    ret = (char*) dbmgr()->get(p->key) ;
  return &ret ;
}
int* rdbm_del_1_svc(rdbm_key_t* p, svc_req*) {
  static int ret ;
  if ( dbmgr()->locked(p->id) == OTHER_LOCK )
    ret = LOCKED ;
  else
    ret = dbmgr()->del(p->key) ? 0 : 1 ;
  return &ret ;
}
int* rdbm_sync_1_svc(rdbm_ident_t* id, svc_req*) {
  static int ret ;
/* use unlock() - just in case we had a lock */
  if ( dbmgr()->unlock(*id) == OTHER_LOCK )
    ret = LOCKED ;
  else
    ret = dbmgr()->sync() ? 0 : 1 ;
  return &ret ;
}
} // end extern "C"
